#
#     File name: model_2d.s 
#
#     Main program to show how to project using missing data method
#     with Poisson errors.  Confidence intervals are included.
#
#    =========================================
#
#    First choose smoothing parameters.  There are two methods
#
#      (a) grid search
#      (b) minimization by ms()
#
#    With grid search we recommend the search is initially
#    on the log scale.  After a few iterations the natural scale
#    can be used.  With ms() we recommend the initial use of grid
#    search to get starting values.  
#
#      ndx.y = 10, ndx.a = 10
#      lambda.a = 0.6, lambda.y = 150, Tr = 41.2, Bic = 4770.5
#
#      as can be found with a grid search
#
Grid.search.year(130, 170, 1, 0.6, a.init, B, v, off, d, P.a, P.y,
                 10^(-8), FALSE, 20, FALSE)
Grid.search.age(0.4, 0.8, 1, 150, a.init, B, v, off, d, P.a, P.y,
                10^(-8), FALSE, 20, FALSE)
#
#      or with ms()
#
ms(~ Calc(a.init, B, v, off, d, P.a, lambda.a, P.y, lambda.y,
          10^(-8), FALSE, 20, TRUE)$Bic,
          start = list(lambda.a = 0.5, lambda.y = 240),
          control = list(maxiter = 10, tol = 0.1))
#
#      Some other results are
#
#      ndx.y = 8, ndx.a = 10
#      lambda.a = 0.5, lambda.y = 90, Tr = 38.9, Bic = 4791.1
#
#      ndx.y = 12, ndx.a = 10
#      lambda.a = 0.5, lambda.y = 330, Tr = 41.2, Bic = 4774.3
#
#      ====================================================
#
#      Set ndx.y = 10, ndx.a = 10 and so
#
lambda.a <- 0.6
lambda.y <- 150
Answer <- Calc(a.init, B, v, off, d, P.a, lambda.a, P.y, lambda.y,
               10^(-8), FALSE, 20, FALSE)
Z.fit <- matrix(Answer$Hazard, nrow = nrow(D), ncol = ncol(D))
#
#     Compute standard errors
#
BtWBplusP <- Answer$BtWBplusP
VBt <- solve(BtWBplusP, t(B))
Var <- apply(t(B) * VBt, 2, sum)
SD.2d <- matrix(sqrt(Var), nrow = nrow(D), ncol = ncol(D))
#
#     Perspective plot - Figure 4
#
par(mfrow = c(1,1))
age.grid <- 11:100
year.grid <- 1947:1999
persp(age.grid, year.grid, Z.fit,
      xlab = "Age", ylab = "Year", zlab = "Log(mortality)")  
#
#     Plot: Ages 35 and 65 - Figure 5
#
select.age <- c(25, 55)
Year <- 1947:1999
par(mfrow = c(1,2))
for (i in 1:2){
  Pick <- select.age[i]
  Dth <- D[Pick, ]
  Exp <- E[Pick, ]
  Mort.obs <- log(Dth/Exp)
  Mort.47.99 <- Z.fit[Pick, ]
  CI.low <- Mort.47.99 - 1.96 * SD.2d[Pick, ]
  CI.up <- Mort.47.99 + 1.96 * SD.2d[Pick, ]
  Y.lim <- c( min(Mort.obs, CI.low), max(Mort.obs, CI.up) )
  plot(Year, Mort.obs, ylim = Y.lim, type = "n", ylab = "Log(mortality)",
       cex = 1)
  points(Year, Mort.obs)
  lines(Year[1:28], Mort.47.99[1:28])
  lines(Year[29:53], Mort.47.99[29:53], lty = 2)
  lines(Year, CI.low, lty = 3)
  lines(Year, CI.up, lty = 3)
  text(1950, Y.lim[1] + 0.1 *(Y.lim[2] - Y.lim[1]),
       paste("Age =", Pick+10), adj = 0)
}
